@@ -350,9 +350,6 @@ pub(crate) fn eval_fn_call_expr(
             return Ok(sum);
         }
         "integrate" => {
-            // Make sure either 3 or 4 arguments were supplied.
-            if expressions.len() < 3 || expressions.len() > 4 {}
-
             return match expressions.len() {
                 3 => calculus::integrate_with_unknown_variable(
                     context,
