@@ -13,6 +13,7 @@ pub mod regular;
 pub use regular::*;
 
 use crate::ast::Expr;
+use crate::radix;
 use lazy_static::lazy_static;
 use std::collections::HashMap;
 use wasm_bindgen::prelude::*;
@@ -52,7 +53,7 @@ lazy_static! {
 #[derive(Clone)]
 pub struct ScientificNotation {
     pub negative: bool,
-    pub(crate) digits: String,
+    pub value: f64,
     pub exponent: i32,
     pub imaginary: bool,
 }
@@ -68,21 +69,13 @@ pub enum ComplexNumberType {
 impl ScientificNotation {
     #[wasm_bindgen(js_name = toString)]
     pub fn to_string(&self) -> String {
-        if self.digits == "" {
-            return String::from("0");
-        }
-
         let sign = if self.negative { "-" } else { "" };
-        let mut digits_and_mul = if self.digits == "1" {
+        let digits_and_mul = if self.value == 1f64 {
             String::new()
         } else {
-            format!("{}*", &self.digits)
+            format!("{}×", format_number(self.value))
         };
 
-        if self.digits.len() > 1 {
-            digits_and_mul.insert(1usize, '.');
-        }
-
         format!(
             "{}{}10^{} {}",
             sign,
@@ -106,31 +99,17 @@ impl KalkNum {
         &self,
         complex_number_type: ComplexNumberType,
     ) -> ScientificNotation {
-        let value_string = match complex_number_type {
-            ComplexNumberType::Real => self.to_string_real(),
-            ComplexNumberType::Imaginary => self.to_string_imaginary(false),
-        }
-        .trim_start_matches("-")
-        .to_string();
-        let trimmed = if value_string.contains(".") {
-            value_string.trim_end_matches("0")
-        } else {
-            &value_string
-        };
         let value = match complex_number_type {
-            ComplexNumberType::Real => &self.value,
-            ComplexNumberType::Imaginary => &self.imaginary_value,
+            ComplexNumberType::Real => self.to_f64(),
+            ComplexNumberType::Imaginary => self.imaginary_to_f64(),
         };
+        let exponent = value.clone().abs().log10().floor() as i32 + 1;
 
         ScientificNotation {
-            negative: value < &0f64,
-            digits: trimmed
-                .to_string()
-                .replace(".", "")
-                .trim_start_matches("0")
-                .to_string(),
+            negative: value < 0f64,
+            value: value / (10f64.powf(exponent as f64 - 1f64) as f64),
             // I... am not sure what else to do...
-            exponent: KalkNum::new(value.clone().abs().log10() + 1f64, "").to_i32(),
+            exponent,
             imaginary: complex_number_type == ComplexNumberType::Imaginary,
         }
     }
@@ -170,12 +149,12 @@ impl KalkNum {
         }
     }
 
-    pub fn to_string_real(&self) -> String {
-        format_number(self.to_f64())
+    pub fn to_string_real(&self, radix: u8) -> String {
+        radix::to_radix_pretty(self.to_f64(), radix)
     }
 
-    pub fn to_string_imaginary(&self, include_i: bool) -> String {
-        let value = format_number(self.imaginary_to_f64());
+    pub fn to_string_imaginary(&self, radix: u8, include_i: bool) -> String {
+        let value = radix::to_radix_pretty(self.imaginary_to_f64(), radix);
         if include_i && value == "1" {
             String::from("i")
         } else if include_i && value == "-1" {
@@ -187,7 +166,7 @@ impl KalkNum {
         }
     }
 
-    pub fn to_string_pretty(&self) -> String {
+    fn to_string_pretty_radix(&self, radix: u8) -> String {
         if let Some(boolean_value) = self.boolean_value {
             return boolean_value.to_string();
         }
@@ -205,12 +184,16 @@ impl KalkNum {
         let sci_notation_real = self.to_scientific_notation(ComplexNumberType::Real);
         let mut adjusted_num = self.clone();
         let result_str = if (-6..8).contains(&sci_notation_real.exponent) || self.value == 0f64 {
-            self.to_string_real()
+            self.to_string_real(radix)
         } else if sci_notation_real.exponent <= -14 {
             adjusted_num.value = KalkNum::from(0f64).value;
             String::from("0")
         } else {
-            sci_notation_real.to_string().trim().to_string()
+            if radix == 10 {
+                sci_notation_real.to_string().trim().to_string()
+            } else {
+                return String::new();
+            }
         };
 
         let sci_notation_imaginary = self.to_scientific_notation(ComplexNumberType::Imaginary);
@@ -218,12 +201,16 @@ impl KalkNum {
             || self.imaginary_value == 0f64
             || self.imaginary_value == 1f64
         {
-            self.to_string_imaginary(true)
+            self.to_string_imaginary(radix, true)
         } else if sci_notation_imaginary.exponent <= -14 {
             adjusted_num.imaginary_value = KalkNum::from(0f64).value;
             String::from("0")
         } else {
-            format!("{}", sci_notation_imaginary.to_string().trim())
+            if radix == 10 {
+                format!("{}", sci_notation_imaginary.to_string().trim())
+            } else {
+                return String::new();
+            }
         };
 
         let mut output = result_str;
@@ -256,7 +243,7 @@ impl KalkNum {
         }
 
         if let Some(estimate) = adjusted_num.estimate() {
-            if estimate != output {
+            if estimate != output && radix == 10 {
                 output.push_str(&format!(" ≈ {}", estimate));
             }
         }
@@ -264,6 +251,17 @@ impl KalkNum {
         output
     }
 
+    pub fn to_string_pretty(&self) -> String {
+        if let Some(other_radix) = self.other_radix {
+            let with_other_radix = self.to_string_pretty_radix(other_radix);
+            if with_other_radix != "" {
+                return format!("{}\n{}", self.to_string_pretty_radix(10), with_other_radix);
+            }
+        }
+
+        self.to_string_pretty_radix(10)
+    }
+
     pub fn is_too_big(&self) -> bool {
         self.value.is_infinite()
     }
@@ -474,13 +472,13 @@ impl KalkNum {
         if let Some(value) = rounded_real {
             output.push_str(&value);
         } else if self.has_real() {
-            output.push_str(&self.to_string_real());
+            output.push_str(&self.to_string_real(10));
         }
 
         let imaginary_value = if let Some(value) = rounded_imaginary {
             Some(value)
         } else if self.has_imaginary() {
-            Some(self.to_string_imaginary(false))
+            Some(self.to_string_imaginary(10, false))
         } else {
             None
         };
@@ -520,7 +518,9 @@ impl KalkNum {
     fn estimate_one_value(&self, complex_number_type: ComplexNumberType) -> Option<String> {
         let (value, value_string) = match complex_number_type {
             ComplexNumberType::Real => (&self.value, self.to_string()),
-            ComplexNumberType::Imaginary => (&self.imaginary_value, self.to_string_imaginary(true)),
+            ComplexNumberType::Imaginary => {
+                (&self.imaginary_value, self.to_string_imaginary(10, true))
+            }
         };
 
         let fract = value.clone().fract().abs();
@@ -679,7 +679,7 @@ impl KalkNum {
     }
 }
 
-fn format_number(input: f64) -> String {
+pub fn format_number(input: f64) -> String {
     let rounded = format!("{:.1$}", input, 10);
     if rounded.contains(".") {
         rounded
@@ -884,7 +884,7 @@ mod tests {
             (0.000000001, -1.0, "10^-9 - i ≈ -i"),
             (10e-17, 1.0, "i"),
             (1.0, 10e-17, "1"),
-            (10e-15, 0.0, "0"),
+            (10e-16, 0.0, "0"),
             (3.00000000004, 0.0, "3"),
         ];
         for (real, imaginary, output) in in_out {
