@@ -1,3 +1,4 @@
+use crate::text_utils::{is_subscript, is_superscript};
 use std::iter::Peekable;
 use std::str;
 use std::str::Chars;
@@ -41,6 +42,7 @@ pub enum TokenKind {
     ClosedBrace,
     Comma,
     Semicolon,
+    Newline,
 
     EOF,
 }
@@ -55,18 +57,23 @@ pub struct Token {
 pub struct Lexer<'a> {
     chars: Peekable<Chars<'a>>,
     index: usize,
+    other_radix: Option<u8>,
 }
 
 impl<'a> Lexer<'a> {
-    pub fn lex(source: &str) -> Vec<Token> {
-        let mut lexer = Lexer {
+    pub fn new(source: &'a str) -> Self {
+        Lexer {
             chars: source.chars().peekable(),
             index: 0,
-        };
+            other_radix: None,
+        }
+    }
+
+    pub fn lex(&mut self) -> Vec<Token> {
         let mut tokens = Vec::new();
 
         loop {
-            let next = lexer.next();
+            let next = self.next();
 
             if let TokenKind::EOF = next.kind {
                 tokens.push(next);
@@ -79,6 +86,10 @@ impl<'a> Lexer<'a> {
         tokens
     }
 
+    pub fn get_other_radix(&self) -> Option<u8> {
+        self.other_radix
+    }
+
     fn next(&mut self) -> Token {
         let eof = build(TokenKind::EOF, "", (self.index, self.index));
         let mut c = if let Some(c) = self.peek() {
@@ -87,7 +98,7 @@ impl<'a> Lexer<'a> {
             return eof;
         };
 
-        while c == ' ' || c == '\t' || c == '\r' || c == '\n' {
+        while c == ' ' || c == '\t' || c == '\r' {
             if let None = self.advance() {
                 return eof;
             }
@@ -111,8 +122,8 @@ impl<'a> Lexer<'a> {
         let token = match c {
             '+' => build(TokenKind::Plus, "", span),
             '-' => build(TokenKind::Minus, "", span),
-            '*' => build(TokenKind::Star, "", span),
-            '/' => build(TokenKind::Slash, "", span),
+            '*' | '×' => build(TokenKind::Star, "", span),
+            '/' | '÷' => build(TokenKind::Slash, "", span),
             '^' => build(TokenKind::Power, "", span),
             '|' => build(TokenKind::Pipe, "", span),
             '⌈' => build(TokenKind::OpenCeil, "", span),
@@ -131,6 +142,7 @@ impl<'a> Lexer<'a> {
             '<' => build(TokenKind::LessThan, "", span),
             ',' => build(TokenKind::Comma, "", span),
             ';' => build(TokenKind::Semicolon, "", span),
+            '\n' => build(TokenKind::Newline, "", span),
             '%' => build(TokenKind::Percent, "", span),
             '\'' => build(TokenKind::Tick, "", span),
             '≠' => build(TokenKind::NotEquals, "", span),
@@ -174,9 +186,11 @@ impl<'a> Lexer<'a> {
     }
 
     fn next_number_literal(&mut self) -> Token {
-        let start = self.index;
+        let mut start = self.index;
         let mut end = start;
         let mut value = String::new();
+        let mut leading_zero = self.peek().unwrap_or(&'\0') == &'0';
+        let mut base = 10u8;
 
         loop {
             let c = if let Some(c) = self.peek() {
@@ -185,7 +199,32 @@ impl<'a> Lexer<'a> {
                 break;
             };
 
-            if !c.is_digit(10) && c != '.' && !c.is_whitespace() || c == '\n' || c == '\r' {
+            // If at the second character and
+            // the first character is a zero,
+            // allow a letter
+            if end - start == 1 && leading_zero {
+                base = match c {
+                    'b' => 2,
+                    'o' => 8,
+                    'x' => 16,
+                    _ => 10,
+                };
+
+                // Don't include eg. 0x in the value
+                if base != 10 {
+                    start += 2;
+                    end += 1;
+                    self.advance();
+                    value.clear();
+                    leading_zero = false;
+                    continue;
+                }
+            }
+
+            if !c.is_digit(base as u32) && c != '.' && c != '_' && !c.is_whitespace()
+                || c == '\n'
+                || c == '\r'
+            {
                 break;
             }
 
@@ -194,6 +233,32 @@ impl<'a> Lexer<'a> {
             self.advance();
         }
 
+        // Subscript unicode symbols after the literal, eg. 11₂
+        let mut base_str = String::new();
+        while crate::text_utils::is_subscript(self.peek().unwrap_or(&'\0')) {
+            base_str.push(*self.peek().unwrap());
+            self.advance();
+        }
+
+        if base_str != "" {
+            base = crate::text_utils::subscript_to_digits(base_str.chars())
+                .parse::<u8>()
+                .unwrap_or(10);
+        }
+
+        if base != 10 {
+            value.push_str(&format!("_{}", base));
+            if let Some(other_radix) = self.other_radix {
+                // Don't bother keeping track of radixes
+                // if several different ones are used
+                if other_radix != base {
+                    self.other_radix = None;
+                }
+            } else {
+                self.other_radix = Some(base);
+            }
+        }
+
         build(TokenKind::Literal, &value, (start, end))
     }
 
@@ -217,7 +282,13 @@ impl<'a> Lexer<'a> {
 
             // Only allow identifiers with a special character to have *one* character. No more.
             // Break the loop if it isn't the first run and the current character is a special character.
-            if end - start > 0 && !(c.is_ascii_alphabetic() || c == '\'' || c == '_') {
+            if end - start > 0
+                && !(c.is_ascii_alphabetic()
+                    || c == '\''
+                    || c == '_'
+                    || is_superscript(&c)
+                    || is_subscript(&c))
+            {
                 break;
             }
 
@@ -236,9 +307,23 @@ impl<'a> Lexer<'a> {
 
         let value = match value.as_ref() {
             "Σ" | "∑" => String::from("sum"),
-            "∫" => String::from("integrate"),
+            "∏" => String::from("prod"),
+            "∫" | "integral" => String::from("integrate"),
+            "sin⁻¹" => String::from("asin"),
+            "cos⁻¹" => String::from("acos"),
+            "tan⁻¹" => String::from("atan"),
+            "cot⁻¹" => String::from("acot"),
+            "cosec⁻¹" => String::from("acosec"),
+            "sec⁻¹" => String::from("asec"),
+            "sinh⁻¹" => String::from("asinh"),
+            "cosh⁻¹" => String::from("acosh"),
+            "tanh⁻¹" => String::from("atanh"),
+            "coth⁻¹" => String::from("acoth"),
+            "cosech⁻¹" => String::from("acosech"),
+            "sech⁻¹" => String::from("asech"),
+            "∛" => String::from("cbrt"),
             "°" => String::from("deg"),
-            _ => value,
+            _ => value, // things like log₂ are handled in the parser
         };
 
         build(kind, &value, (start, end))
@@ -267,8 +352,8 @@ fn is_valid_identifier(c: Option<&char>) -> bool {
         match c {
             '+' | '-' | '/' | '*' | '%' | '^' | '!' | '(' | ')' | '=' | '.' | ',' | ';' | '|'
             | '⌊' | '⌋' | '⌈' | '⌉' | '[' | ']' | '{' | '}' | 'π' | '√' | 'τ' | 'ϕ' | 'Γ' | '<'
-            | '>' | '≠' | '≥' | '≤' => false,
-            _ => !c.is_digit(10),
+            | '>' | '≠' | '≥' | '≤' | '×' | '÷' | '\n' => false,
+            _ => !c.is_digit(10) || is_superscript(c) || is_subscript(c),
         }
     } else {
         false
@@ -293,7 +378,7 @@ mod tests {
     #[test]
     #[wasm_bindgen_test]
     fn test_token_kinds() {
-        let tokens = Lexer::lex("+-*/%^()|=!,");
+        let tokens = Lexer::new("+-*/%^()|=!,").lex();
         let expected = vec![
             TokenKind::Plus,
             TokenKind::Minus,
@@ -316,7 +401,7 @@ mod tests {
     #[test]
     #[wasm_bindgen_test]
     fn test_brackets() {
-        let tokens = Lexer::lex("[1 < 2]");
+        let tokens = Lexer::new("[1 < 2]").lex();
         let expected = vec![
             TokenKind::OpenBracket,
             TokenKind::Literal,
@@ -336,7 +421,7 @@ mod tests {
         let test_cases = vec![" ", "     ", "test ", " test     "];
 
         for input in test_cases {
-            let tokens = Lexer::lex(input);
+            let tokens = Lexer::new(input).lex();
 
             if regex::Regex::new(r"^\s*$").unwrap().is_match(input) {
                 let expected = vec![TokenKind::EOF];
@@ -352,7 +437,7 @@ mod tests {
     #[test_case("24")]
     #[test_case("56.4")]
     fn test_number_literal(input: &str) {
-        let tokens = Lexer::lex(input);
+        let tokens = Lexer::new(input).lex();
         let expected = vec![TokenKind::Literal, TokenKind::EOF];
 
         assert_eq!(&tokens[0].value, input);
@@ -362,7 +447,7 @@ mod tests {
     #[test_case("x")]
     #[test_case("xy")]
     fn test_identifier(input: &str) {
-        let tokens = Lexer::lex(input);
+        let tokens = Lexer::new(input).lex();
         let expected = vec![TokenKind::Identifier, TokenKind::EOF];
 
         assert_eq!(&tokens[0].value, input);
@@ -371,7 +456,7 @@ mod tests {
 
     #[test]
     fn test_function_call() {
-        let tokens = Lexer::lex("f(x)");
+        let tokens = Lexer::new("f(x)").lex();
         let expected = vec![
             TokenKind::Identifier,
             TokenKind::OpenParenthesis,
