@@ -1,9 +1,13 @@
 pub mod ast;
+mod calculus;
+mod integration_testing;
 mod interpreter;
 mod inverter;
 pub mod kalk_num;
 mod lexer;
 pub mod parser;
 mod prelude;
+mod radix;
 mod symbol_table;
 mod test_helpers;
+pub mod text_utils;
