@@ -33,6 +33,7 @@ pub struct Context {
     is_in_integral: bool,
     current_function: Option<String>,
     current_function_parameters: Option<Vec<String>>,
+    other_radix: Option<u8>,
 }
 
 #[wasm_bindgen]
@@ -53,6 +54,7 @@ impl Context {
             is_in_integral: false,
             current_function: None,
             current_function_parameters: None,
+            other_radix: None,
         };
 
         parse(&mut context, crate::prelude::INIT).unwrap();
@@ -114,6 +116,7 @@ pub enum CalcError {
     UnableToSolveEquation,
     UnableToOverrideConstant(String),
     UnableToParseExpression,
+    UnrecognizedBase,
     Unknown,
 }
 
@@ -142,6 +145,7 @@ impl ToString for CalcError {
             CalcError::UnableToParseExpression => format!("Unable to parse expression."),
             CalcError::UnableToSolveEquation => format!("Unable to solve equation."),
             CalcError::UnableToOverrideConstant(name) => format!("Unable to override constant: '{}'.", name),
+            CalcError::UnrecognizedBase => format!("Unrecognized base."),
             CalcError::Unknown => format!("Unknown error."),
         }
     }
@@ -174,17 +178,25 @@ pub fn eval(
             None
         },
     );
-    interpreter.interpret(statements)
+    let result = interpreter.interpret(statements);
+    if let Ok(Some(mut num)) = result {
+        num.other_radix = context.other_radix;
+        Ok(Some(num))
+    } else {
+        result
+    }
 }
 
 /// Parse expressions/declarations and return a syntax tree.
 ///
 /// `None` will be returned if the last statement is a declaration.
 pub fn parse(context: &mut Context, input: &str) -> Result<Vec<Stmt>, CalcError> {
-    context.tokens = Lexer::lex(input);
+    let mut lexer = Lexer::new(input);
+    context.tokens = lexer.lex();
     context.pos = 0;
     context.parsing_unit_decl = false;
     context.unit_decl_base_unit = None;
+    context.other_radix = lexer.get_other_radix();
 
     let mut statements: Vec<Stmt> = Vec::new();
     while !is_at_end(context) {
@@ -193,6 +205,8 @@ pub fn parse(context: &mut Context, input: &str) -> Result<Vec<Stmt>, CalcError>
         if match_token(context, TokenKind::Semicolon) {
             advance(context);
         }
+
+        skip_newlines(context);
     }
 
     Ok(statements)
@@ -271,6 +285,7 @@ fn parse_identifier_stmt(context: &mut Context) -> Result<Stmt, CalcError> {
 
 fn parse_piecewise(context: &mut Context) -> Result<Expr, CalcError> {
     advance(context);
+    skip_newlines(context);
 
     let mut pieces = Vec::new();
     let mut reached_otherwise = false;
@@ -302,10 +317,18 @@ fn parse_piecewise(context: &mut Context) -> Result<Expr, CalcError> {
             return Err(CalcError::ExpectedIf);
         }
 
-        advance(context);
-        previous(context).kind == TokenKind::Semicolon && !reached_otherwise
+        if match_token(context, TokenKind::Semicolon) {
+            advance(context);
+        }
+        skip_newlines(context);
+
+        (previous(context).kind == TokenKind::Semicolon
+            || previous(context).kind == TokenKind::Newline)
+            && !reached_otherwise
     } {}
 
+    advance(context);
+
     Ok(Expr::Piecewise(pieces))
 }
 
@@ -589,8 +612,19 @@ fn parse_group_fn(context: &mut Context) -> Result<Expr, CalcError> {
 
 fn parse_identifier(context: &mut Context) -> Result<Expr, CalcError> {
     let identifier = Identifier::from_full_name(&advance(context).value);
+
+    let mut log_base = None;
+    if identifier.full_name.starts_with("log") {
+        if let Some(c) = identifier.full_name.chars().nth(3) {
+            if crate::text_utils::is_subscript(&c) {
+                log_base = Some(Expr::Literal(get_base(&identifier.full_name)? as f64));
+            }
+        }
+    }
+
     let exists_as_fn = context.symbol_table.contains_fn(&identifier.pure_name)
-        || context.current_function.as_ref() == Some(&identifier.pure_name);
+        || context.current_function.as_ref() == Some(&identifier.pure_name)
+        || log_base.is_some();
 
     // Eg. sqrt64
     if exists_as_fn
@@ -602,6 +636,14 @@ fn parse_identifier(context: &mut Context) -> Result<Expr, CalcError> {
         } else {
             parse_factor(context)?
         };
+
+        if let Some(log_base) = log_base {
+            return Ok(Expr::FnCall(
+                Identifier::from_full_name("log"),
+                vec![parameter, log_base],
+            ));
+        }
+
         return Ok(Expr::FnCall(identifier, vec![parameter]));
     }
 
@@ -637,6 +679,11 @@ fn parse_identifier(context: &mut Context) -> Result<Expr, CalcError> {
             context.is_in_integral = false;
         }
 
+        if let Some(log_base) = log_base {
+            parameters.push(log_base);
+            return Ok(Expr::FnCall(Identifier::from_full_name("log"), parameters));
+        }
+
         return Ok(Expr::FnCall(identifier, parameters));
     }
 
@@ -828,14 +875,36 @@ fn is_at_end(context: &Context) -> bool {
     context.pos >= context.tokens.len() || peek(context).kind == TokenKind::EOF
 }
 
+fn skip_newlines(context: &mut Context) {
+    while match_token(context, TokenKind::Newline) {
+        advance(context);
+    }
+}
+
 fn string_to_num(value: &str) -> Result<f64, CalcError> {
-    if let Ok(result) = value.replace(" ", "").parse::<f64>() {
+    let base = get_base(value)?;
+    if let Some(result) = crate::radix::parse_float_radix(&value.replace(" ", ""), base) {
         Ok(result)
     } else {
         Err(CalcError::InvalidNumberLiteral(value.into()))
     }
 }
 
+fn get_base(value: &str) -> Result<u8, CalcError> {
+    let underscore_pos = if let Some(i) = value.find('_') {
+        i
+    } else {
+        return Ok(10);
+    };
+
+    let subscript = value.chars().skip(underscore_pos + 1usize);
+    if let Some(base) = crate::text_utils::parse_subscript(subscript) {
+        Ok(base)
+    } else {
+        Err(CalcError::UnrecognizedBase)
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
